#include "Arduino.h"
#include "_74595_sevenSegment.h"
#include "FontMap7Segment.h"


sevenSegment::sevenSegment(byte data,byte latch,byte clock, byte digits)
{
	pinMode(data,OUTPUT);
	pinMode(latch,OUTPUT);
	pinMode(clock,OUTPUT);
	
	_data=data;
	_latch=latch;
	_clock=clock;
	_digits=digits;
}


void sevenSegment::setcolumns(unsigned char column)
{
  for (_Position = _digits; _Position > -1; _Position--)
  {
    if (column == _Position)
    {
      digitalWrite(_data, HIGH);
    }
    else
    {
      digitalWrite(_data, LOW);
    }
    digitalWrite(_clock, HIGH);
    digitalWrite(_clock, LOW);
  }
}


void sevenSegment::setdata(unsigned char Data)
{
  for (_Position = 0; _Position < 8; _Position++)
  {
    if (Data & 128)
    {
      Data -= 128;
      digitalWrite(_data, HIGH);
    }
    else
    {
      digitalWrite(_data, LOW);
    }
    Data = Data << 1;
    digitalWrite(_clock, HIGH);
    digitalWrite(_clock, LOW);
  }
}

void sevenSegment::sevenSegmentClr() //clear buffer
{
  for (_address = 0; _address < _digits; _address++)  // Empty display buffer, same size as buffer size
    _screenBuffer[_address] = 0;
}



void sevenSegment::sevenSegmentDraw()
{
  setcolumns(_displayPointer);            //column scanning
  setdata(_screenBuffer[_displayPointer]);    //bit banging shift registers
  digitalWrite(_latch, LOW);
  digitalWrite(_latch, HIGH);
  if (++_displayPointer == _digits)          //32 LED row sections in total
  {
    _displayPointer = 0;
  }
}




void sevenSegment::sevenSegmentSegment(signed char x, signed char y, byte condition)
{
  if (x < 0 || y < 0)
  {
    return;  // outside drawing limits negative
  }

  if (x > _digits-1|| y > 7)
  {
    return;  // outside drawing limits positive, changed for columns used
  }

  _pix = _power[y];
  _mask = _screenBuffer[x];  // get exsisting data
  if (condition == 2)
    _pix ^= _mask;  // XOR data to screen
  if (condition == 1)
  {
    _pix = ~_pix;
    _pix &= _mask;  // AND data to screen
  }

  if (condition == 0)
    _pix |= _mask;  // OR data to screen
  _screenBuffer[x] = _pix;  // apply changes
}


void sevenSegment::sevenSegmentValue(unsigned char ch, signed char x, signed char y)
{
  _disp = sevenSegmentFont[ch];   //look for character value from fontmap, like character 0, ascii value is 0, ASCII table 48, and at start of fontmap.

  for (_y1 = 0; _y1 < 8; _y1++)  // eight pixels
  {
    _disp2 = _disp & _power[_y1];
    if (_disp2 > 0)
    {
      sevenSegmentSegment(x, y + _y1, 0);  // OR the pixel to the display buffer
    }
  }
}

void sevenSegment::sevenSegmentText(const char* ch, signed char x, signed char y)
{
  while (*ch )
  {
    sevenSegmentValue(*ch++, x, y);  // write a string to the display buffer
    x += 1;  //jumps one columns at a time (x=x+1)
  }
}